/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2014, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "blocks.h"


namespace Ezc
{


void Blocks::Insert(const std::wstring & name, const Item & item)
{
	blocks_tab[name] = item;
}


Blocks::Iterator Blocks::Find(const std::wstring & name)
{
	return blocks_tab.find(name);
}


Blocks::Iterator Blocks::Begin()
{
	return blocks_tab.begin();
}


Blocks::Iterator Blocks::End()
{
	return blocks_tab.end();
}


size_t Blocks::Size() const
{
	return blocks_tab.size();
}


void Blocks::Clear()
{
	blocks_tab.clear();
}


void Blocks::ClearCache()
{
	BlocksTable::iterator i = blocks_tab.begin();

	for( ; i != blocks_tab.end() ; ++i)
		i->second.ClearCache();
}



void Blocks::CacheBlocks(Blocks & blocks)
{
	BlocksTable::iterator i = blocks_tab.begin();

	for( ; i != blocks_tab.end() ; ++i)
		Cache(blocks, i->second);
}



} // namespace
