/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2014-2015, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_ezc_blocks
#define headerfile_ezc_blocks

#include <string>
#include <map>
#include "item.h"
#include "cache.h"


namespace Ezc
{



class Blocks
{
public:

	typedef std::map<std::wstring, Item> BlocksTable;
	typedef BlocksTable::iterator Iterator;

	void Insert(const std::wstring & name, const Item & item);

	Iterator Find(const std::wstring & name);

	Iterator Begin();
	Iterator End();
	size_t   Size() const;
	void     Clear();

	template<class StreamType> void CacheObjects(Objects<StreamType> & obj);
	template<class StreamType> void CacheFunctions(Functions<StreamType> & fun);

	void CacheBlocks(Blocks & blocks);
	void ClearCache();

private:

	BlocksTable blocks_tab;

};


template<class StreamType>
void Blocks::CacheObjects(Objects<StreamType> & obj)
{
	BlocksTable::iterator i = blocks_tab.begin();

	for( ; i != blocks_tab.end() ; ++i)
		Cache(obj, i->second);
}


template<class StreamType>
void Blocks::CacheFunctions(Functions<StreamType> & fun)
{
	BlocksTable::iterator i = blocks_tab.begin();

	for( ; i != blocks_tab.end() ; ++i)
		Cache(fun, i->second);
}



} // namespace

#endif
