/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2007-2015, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_ezc_functions
#define headerfile_ezc_functions

#include <map>
#include "utf8/utf8.h"
#include "funinfo.h"



namespace Ezc
{




// functions or variables
template<class StreamType>
class Functions
{
public:

	typedef void (*UserFunction)(FunInfo<StreamType> &);
	typedef std::map<std::wstring, UserFunction> FunctionsTable;
	typedef typename FunctionsTable::iterator Iterator;

	void Insert(const char * key,         UserFunction ufunction);
	void Insert(const std::string & key,  UserFunction ufunction);
	void Insert(const wchar_t * key,      UserFunction ufunction);
	void Insert(const std::wstring & key, UserFunction ufunction);

	Iterator Find(const std::wstring & key);

	Iterator Begin();
	Iterator End();
	size_t   Size() const;
	void     Clear();
	

private:

	FunctionsTable functions_tab;
	std::wstring temp_key;

};






template<class StreamType>
void Functions<StreamType>::Insert(const char * key, UserFunction ufunction)
{
	PT::UTF8ToWide(key, temp_key);
	functions_tab[temp_key] = ufunction;
	temp_key.clear();
}


template<class StreamType>
void Functions<StreamType>::Insert(const std::string & key, UserFunction ufunction)
{
	Insert(key.c_str(), ufunction);
}


template<class StreamType>
void Functions<StreamType>::Insert(const wchar_t * key, UserFunction ufunction)
{
	temp_key = key;
	functions_tab[temp_key] = ufunction;
	temp_key.clear();
}


template<class StreamType>
void Functions<StreamType>::Insert(const std::wstring & key, UserFunction ufunction)
{
	functions_tab[key] = ufunction;
}








template<class StreamType>
typename Functions<StreamType>::Iterator Functions<StreamType>::Find(const std::wstring & key)
{
	return functions_tab.find(key);
}



template<class StreamType>
void Functions<StreamType>::Clear()
{
	functions_tab.clear();
}


template<class StreamType>
typename Functions<StreamType>::Iterator Functions<StreamType>::Begin()
{
	return functions_tab.begin();
}


template<class StreamType>
typename Functions<StreamType>::Iterator Functions<StreamType>::End()
{
	return functions_tab.end();
}


template<class StreamType>
size_t Functions<StreamType>::Size() const
{
	return functions_tab.size();
}



} // namespace Ezc


#endif
