/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2007-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "item.h"



namespace Ezc
{





Item * Item::AddItem(const Item * porg)
{
Item * pitem;
	
	if( porg )
		pitem = new Item(*porg);
	else
		pitem = new Item();

	item_tab.push_back(pitem);

return pitem;
}



Item * Item::AddItem(const Item & porg)
{
	return AddItem(&porg);
}



void Item::ClearItems()
{
	std::vector<Item*>::iterator i = item_tab.begin();

	for( ; i != item_tab.end() ; ++i )
		delete *i;

	item_tab.clear();
}




void Item::Clear()
{
	ClearItems();

	text.clear();
	file_name.clear();
	has_function = false;
	function.Clear();
	type = item_none;
}



Item::Type Item::LastItemType()
{
	if( item_tab.empty() )
		return item_none;

return item_tab.back()->type;
}



void Item::DeleteLastItem()
{
	if( item_tab.empty() )
		return;

	delete item_tab.back();
	item_tab.pop_back();
}



Item::Item()
{
	type = item_none;
	has_function = false;
}



Item::Item(const Item & i) : text(i.text), file_name(i.file_name), function(i.function)
{
	type = i.type;
	has_function = i.has_function;
	CopyItemTable(i);
}



Item & Item::operator=(const Item & i)
{
	type = i.type;
	text = i.text;
	file_name    = i.file_name;
	has_function = i.has_function;
	function     = i.function;

	ClearItems();
	CopyItemTable(i);

return *this;
}



void Item::CopyItemTable(const Item & item)
{
	std::vector<Item*>::const_iterator i = item.item_tab.begin();

	for( ; i != item.item_tab.end() ; ++i)
		AddItem(*i);
}



Item::~Item()
{
	ClearItems();
}


void Item::ClearCache(Item::Function & function)
{
	function.base_obj = 0;
	function.method_index = -1;
	function.fun_cache  = 0;
	function.item_block = 0;

	for(size_t i=0 ; i<function.parameters.size() ; ++i)
		ClearCache(*function.parameters[i]);
}


void Item::ClearCache()
{
	ClearCache(function);

	for(size_t i = 0; i < item_tab.size() ; ++i)
		item_tab[i]->ClearCache();
}




} // namespace Ezc

