/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2015, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_ezc_outstreams
#define headerfile_ezc_outstreams

#include <string>
#include <vector>
#include <map>


namespace Ezc
{


template<class StreamType>
class OutStreams
{
public:

	typedef std::map<std::wstring, StreamType*> StreamsMap;
	typedef std::vector<StreamType*> StreamsTab;

	StreamsMap streams_map;
	StreamsTab streams_tab;

	void ResizeTab(size_t len);

	void ClearMap();
	void ClearTab();
	
	~OutStreams();
	OutStreams();
	OutStreams(const OutStreams<StreamType> & o);
	OutStreams<StreamType> & operator=(const OutStreams<StreamType> & o);
	

};


template<class StreamType>
OutStreams<StreamType>::~OutStreams()
{
	ClearTab();
}

template<class StreamType>
OutStreams<StreamType>::OutStreams()
{
}



template<class StreamType>
OutStreams<StreamType>::OutStreams(const OutStreams<StreamType> & o)
{
	// we do not copy streams but creating new ones
	ResizeTab(o.streams_tab.size());
}


template<class StreamType>
OutStreams<StreamType> & OutStreams<StreamType>::operator=(const OutStreams<StreamType> & o)
{
	// we do not copy streams but creating new ones
	streams_map.clear();
	ResizeTab(o.streams_tab.size());

return *this;
}


template<class StreamType>
void OutStreams<StreamType>::ClearTab()
{
	for(size_t i=0 ; i<streams_tab.size() ; ++i)
		delete streams_tab[i];

	streams_tab.clear();
	streams_map.clear();
}


template<class StreamType>
void OutStreams<StreamType>::ResizeTab(size_t len)
{
	if( streams_tab.size() != len )
	{
		if( streams_tab.size() < len )
		{
			size_t i = streams_tab.size();
			streams_tab.resize(len);

			for( ; i<streams_tab.size() ; ++i)
				streams_tab[i] = new StreamType();
		}
		else
		{
			streams_map.clear();

			for(size_t i=len ; i<streams_tab.size() ; ++i)
				delete streams_tab[i];
			
			streams_tab.resize(len);
		}
	}
}


template<class StreamType>
void OutStreams<StreamType>::ClearMap()
{
	typename StreamsMap::iterator i;

	for(i=streams_map.begin() ; i != streams_map.end() ; ++i)
	{
		StreamType & str = *(i->second);

		#ifdef EZC_HAS_SPECIAL_STREAM
			str.Clear();
		#else
			str.str(L"");
		#endif

	}

	streams_map.clear();
}






} // namespace

#endif
